import org.junit.Assert;
import org.junit.Test;

import java.lang.reflect.Field;
import java.util.*;

import static org.hamcrest.CoreMatchers.is;
import static org.junit.Assert.*;

public class Aufgabe1Test {

    @Test
    public void testPotionInitIngredients() throws Exception {
        Queue<Ingredient> ingr = new LinkedList<Ingredient>();
        ingr.offer(new Ingredient("Bergblume"));
        ingr.offer(new Ingredient("Weizen"));
        Potion healing = new Potion(ingr);
        Field field = Potion.class.
                getDeclaredField("ingredients");
        field.setAccessible(true);
        Collection<Ingredient> colA = (Collection<Ingredient>) field.get(healing);
        List<Ingredient> colB = new LinkedList<>();
        colB.add(new Ingredient("Bergblume"));
        colB.add(new Ingredient("Weizen"));
        Assert.assertTrue(colA.containsAll(colB) && colB.containsAll(colA));
        Assert.assertThat(colA.size(), is(2));
    }

    @Test
    public void testPotionInitStirReady() throws Exception {
        Queue<Ingredient> ingr = new LinkedList<Ingredient>();
        ingr.offer(new Ingredient("Bergblume"));
        ingr.offer(new Ingredient("Weizen"));
        Potion healing = new Potion(ingr);
        assertThat(healing.ready(), is(false));
        healing.stir();
        assertThat(healing.ready(), is(false));
        healing.stir();
        assertThat(healing.ready(), is(true));
    }

    @Test
    public void testPotionInitPower() throws Exception {
        Queue<Ingredient> ingr = new LinkedList<Ingredient>();
        ingr.offer(new Ingredient("Bergblume"));
        ingr.offer(new Ingredient("Weizen"));
        Potion healing = new Potion(ingr);
        assertThat(healing.power(), is(15));
    }

    @Test
    public void testPotionInitStirPower() throws Exception {
        Queue<Ingredient> ingr = new LinkedList<Ingredient>();
        ingr.offer(new Ingredient("Bergblume"));
        ingr.offer(new Ingredient("Weizen"));
        Potion healing = new Potion(ingr);
        assertThat(healing.power(), is(15));
        healing.stir();
        assertThat(healing.power(), is(15));
        healing.stir();
        assertThat(healing.power(), is(30));
    }

    @Test
    public void testPotionMixWithIngredients() throws Exception {
        Potion healing1 = new Potion(new String[]{"Bergblume", "Weizen"});
        Potion healing2 = new Potion(new String[]{"Bergblume","Säbelzahntigerauge","Knochenmehl"});

        Potion healing3 = healing1.mixWith(healing2);


        Field field = Potion.class.
                getDeclaredField("ingredients");
        field.setAccessible(true);

        Collection<Ingredient> colA = (Collection<Ingredient>) field.get(healing3);
        List<Ingredient> colB = new LinkedList<>();
        colB.add(new Ingredient("Bergblume"));
        colB.add(new Ingredient("Säbelzahntigerauge"));
        colB.add(new Ingredient("Weizen"));
        colB.add(new Ingredient("Knochenmehl"));
        Assert.assertTrue(colA.containsAll(colB) && colB.containsAll(colA));
        Assert.assertThat(colA.size(), is(4));

    }

    @Test
    public void testPotionMixWithStired() throws Exception {
        Potion healing1 = new Potion(new String[]{"Bergblume", "Weizen"});
        Potion healing2 = new Potion(new String[]{"Bergblume","Säbelzahntigerauge","Knochenmehl"});

        Potion healing3 = healing1.mixWith(healing2);


        assertThat(healing3.ready(), is(true));

    }

    @Test
    public void testCabinet() throws Exception {
        Potion healing1 = new Potion(new String[]{"Bergblume", "Weizen"});
        Potion healing2 = new Potion(new String[]{"Bergblume","Säbelzahntigerauge"});
        Potion invisibility = new Potion(new String[]{"Mondfalterflügel","Vampirstaub"});

        Cabinet cabinet = new Cabinet();

        cabinet.put("Heilung", healing1);
        cabinet.put("Heilung", healing2);
        cabinet.put("Unsichtbarkeit", invisibility);

        Field field = Cabinet.class.
                getDeclaredField("potions");
        field.setAccessible(true);

        Map<String, Potion> mapA = (Map<String, Potion>) field.get(cabinet);
        Potion potionA = ((Potion) mapA.get("Heilung"));

        field = Potion.class.
                getDeclaredField("ingredients");
        field.setAccessible(true);

        List<Ingredient> colA = (List<Ingredient>) field.get(potionA);
        List<Ingredient> colB = new LinkedList<>();
        colB.add(new Ingredient("Bergblume"));
        colB.add(new Ingredient("Säbelzahntigerauge"));
        colB.add(new Ingredient("Weizen"));
        Assert.assertTrue(colA.containsAll(colB) && colB.containsAll(colA));
        Assert.assertThat(colA.size(), is(3));

        potionA = ((Potion) mapA.get("Unsichtbarkeit"));
        colA = (List<Ingredient>) field.get(potionA);
        colB = new LinkedList<>();
        colB.add(new Ingredient("Mondfalterflügel"));
        colB.add(new Ingredient("Vampirstaub"));
        Assert.assertTrue(colA.containsAll(colB) && colB.containsAll(colA));
        Assert.assertThat(colA.size(), is(2));
    }

    @Test
    public void testCabinetStirBeforeStore() throws Exception {
        Potion invisibility = new Potion(new String[]{"Mondfalterflügel","Vampirstaub"});

        Cabinet cabinet = new Cabinet();
        cabinet.put("Unsichtbarkeit", invisibility);

        Field field = Cabinet.class.
                getDeclaredField("potions");
        field.setAccessible(true);

        Map<String, Potion> mapA = (Map<String, Potion>) field.get(cabinet);
        Potion potionA = ((Potion) mapA.get("Unsichtbarkeit"));

        Assert.assertThat(potionA.ready(), is(true));
    }

}